
//
// This source file is part of appleseed.
// Visit http://appleseedhq.net/ for additional information and resources.
//
// This software is released under the MIT license.
//
// Copyright (c) 2012-2013 Esteban Tovagliari, Jupiter Jazz Limited
// Copyright (c) 2014 Esteban Tovagliari, The appleseedhq Organization
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

// Has to be first, to avoid redefinition warnings.
#include "bind_auto_release_ptr.h"

// appleseed.python headers.
#include "bind_typed_entity_containers.h"
#include "dict2dict.h"
#include "unaligned_transformd44.h"

// appleseed.renderer headers.
#include "renderer/api/object.h"
#include "renderer/modeling/scene/objectinstance.h"

// appleseed.foundation headers.
#include "foundation/utility/searchpaths.h"

namespace bpy = boost::python;
using namespace foundation;
using namespace renderer;

namespace detail
{
    void bpy_list_to_string_array(const bpy::list& l, StringArray& strings)
    {
        strings.clear();

        for (bpy::ssize_t i = 0, e = bpy::len(l); i < e; ++i)
        {
            bpy::extract<const char*> ex(l[i]);
            if (!ex.check())
            {
                PyErr_SetString(PyExc_TypeError, "Incompatible type. Only strings.");
                bpy::throw_error_already_set();
            }

            strings.push_back(ex());
        }
    }

    auto_release_ptr<ObjectInstance> create_obj_instance_with_back_mat(const std::string& name,
                                                                       const bpy::dict& params,
                                                                       const std::string& object_name,
                                                                       const UnalignedTransformd44& transform,
                                                                       const bpy::dict& front_material_mappings,
                                                                       const bpy::dict& back_material_mappings)
    {
        return ObjectInstanceFactory::create(name.c_str(),
                                             bpy_dict_to_param_array(params),
                                             object_name.c_str(),
                                             transform.as_foundation_transform(),
                                             bpy_dict_to_dictionary(front_material_mappings).strings(),
                                             bpy_dict_to_dictionary(back_material_mappings).strings());
    }

    auto_release_ptr<ObjectInstance> create_obj_instance(const std::string& name,
                                                         const bpy::dict& params,
                                                         const std::string& object_name,
                                                         const UnalignedTransformd44& transform,
                                                         const bpy::dict& front_material_mappings)
    {
        return create_obj_instance_with_back_mat(name,
                                                 params,
                                                 object_name,
                                                 transform,
                                                 front_material_mappings,
                                                 bpy::dict());
    }

    UnalignedTransformd44 obj_inst_get_transform(const ObjectInstance* obj)
    {
        return UnalignedTransformd44(obj->get_transform());
    }
}

void bind_object()
{
    bpy::class_<Object, auto_release_ptr<Object>, bpy::bases<Entity>, boost::noncopyable>("Object", bpy::no_init)
        .def("compute_local_bbox", &Object::compute_local_bbox);

    bind_typed_entity_vector<Object>("ObjectContainer");

    bpy::class_<ObjectInstance, auto_release_ptr<ObjectInstance>, bpy::bases<Entity>, boost::noncopyable>("ObjectInstance", bpy::no_init)
        .def("__init__", bpy::make_constructor(detail::create_obj_instance))
        .def("__init__", bpy::make_constructor(detail::create_obj_instance_with_back_mat))
        .def("get_object", &ObjectInstance::get_object, bpy::return_value_policy<bpy::reference_existing_object>())
        .def("get_transform", &detail::obj_inst_get_transform)
        .def("compute_parent_bbox", &ObjectInstance::compute_parent_bbox);

    bind_typed_entity_vector<ObjectInstance>("ObjectInstanceContainer");
}
